#! /bin/sh
# the next line restarts using tclsh \
exec `which tclsh` $0 ${1+"$@"}

if {0} {
hostup? client for LCDd
Written 2006 by Jannis Achstetter <jannis_achstetter@web.de>

Changelog:

*v0.1:
  - Initial release

/Changelog

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
}

## -=- Settings start here -=-

# Delay between updates in ms:
set updatetime 500
set lcddport 13666

array set hosts {
  1 "kripserver.net"
  2 "192.168.189.201"
  3 "192.168.189.222"
}

## -=- Settings end -=-
## You actually don't need to change anything below this line

proc connectlcd {} {
  global lcddport lcdsock

  if {[catch {set lcdsock [socket 127.0.0.1 $lcddport]}]} {
    puts "Error connecting to LCDd on 127.0.0.1:$lcddport"
    cleanexit 1
  }
  fileevent $lcdsock readable [list lcdsockread]

  puts $lcdsock "hello"
  flush $lcdsock
}

proc lcdsockread {} {
  global lcdsock lcdversion lcdwid lcdhgt screenactive

  gets $lcdsock line

  if {$line == ""} {
    puts "LCDd died, so we'll suicide! Bye, bye! BOOM!"
    cleanexit 0
  }

  if {[string match {connect LCD*} $line]} {
    set list [split $line]
    set lcdversion [lindex $line 2]
    set lcdprot [lindex $line 4]
    set lcdwid [lindex $line 7]
    set lcdhgt [lindex $line 9]
    set lcdcwid [lindex $line 11]
    set lcdchgt [lindex $line 13]
    puts $lcdsock "client_set name {hostup}"
    puts $lcdsock "screen_add hostup"
    puts $lcdsock "screen_set hostup name hostup"
    puts $lcdsock "widget_add hostup title title"
    puts $lcdsock "widget_add hostup line1 string"
    puts $lcdsock "widget_add hostup line2 string"
    puts $lcdsock "widget_add hostup line3 string"
    puts $lcdsock "widget_set hostup title {HostUp?}"
    flush $lcdsock
  }
  if {[string match {ignore*} $line]} {
    set screenactive 0
  }
  if {[string match {listen*} $line]} {
    set screenactive 1
  }
}

proc updatehostup {} {
  global lcdsock lcdwid lcdhgt hosts stats screenactive updatetime

  if {$screenactive == 1} {
    array set stats {
      1 ""
      2 ""
      3 ""
    }
    if {[lindex [array get hosts 1] 1] != ""} {
      if {[catch {set ret [exec ping -c 1 [lindex [array get hosts 1] 1]]}]} {
        set ret "ERR"
      }
      set list [split $ret \x0A]
      if {$ret == "ERR"} {
        array set stats [list 1 "ERR"]
      } else {
        foreach i $list {
          if {[string match *time=* $i]} {
            array set stats [list 1 [lindex [split $i =] 3]]
          }
        }
      }
    } else {
      array set stats [list 1 "---"]
    }
    if {[lindex [array get hosts 2] 1] != ""} {
      if {[catch {set ret [exec ping -c 1 [lindex [array get hosts 2] 1]]}]} {
        set ret "ERR"
      }
      set list [split $ret \x0A]
      if {$ret == "ERR"} {
        array set stats [list 2 "ERR"]
      } else {
        foreach i $list {
          if {[string match *time=* $i]} {
            array set stats [list 2 [lindex [split $i =] 3]]
          }
        }
      }
    } else {
      array set stats [list 2 "---"]
    }
    if {[lindex [array get hosts 3] 1] != ""} {
      if {[catch {set ret [exec ping -c 1 [lindex [array get hosts 3] 1]]}]} {
        set ret "ERR"
      }
      set list [split $ret \x0A]
      if {$ret == "ERR"} {
        array set stats [list 3 "ERR"]
      } else {
        foreach i $list {
          if {[string match *time=* $i]} {
            array set stats [list 3 [lindex [split $i =] 3]]
          }
        }
      }
    } else {
      array set stats [list 3 "---"]
    }
    foreach i [array names stats] {
      if {[lindex [array get stats $i] 1] != "---"} {
        set hname [string range [lindex [array get hosts $i] 1] 0 14]
        append hname " "
        set linestring $hname
        set stat [string map { " " ""} [lindex [array get stats $i] 1]]
        if {$stat != "ERR"} {
          set precom [lindex [split $stat .] 0]
          set postcom [lindex [split [lindex [split $stat .] 1] m] 0]
          set stat $precom\.[string range $postcom 0 0]ms
        }
        set count [expr $lcdwid - [string length $linestring] - [string length $stat]]
        for {set x 0} {$x<$count} {incr x} {
          append linestring " "
        }
        append linestring $stat
        puts $lcdsock "widget_set hostup line$i 1 [expr $i+1] {$linestring}"
      }
      flush $lcdsock
    }
  }
  after $updatetime {updatehostup}
}

proc cleanexit {exitcode} {
  global lcdsock
  catch {close $lcdsock}
  exit $exitcode
}

connectlcd
#we have to wait until we know how large the LCD is and the screen is active
vwait lcdwid
vwait screenactive
updatehostup

vwait forever
